﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEditor;
//using UnityEngine.Windows;
using System.IO;
using System;

public enum TextureValue
{
    Int8,
    UInt8,
    Float
}

public class LoadObject : MonoBehaviour
{
    public GameObject lodLevel0;
    public GameObject lodLevel1;
    public GameObject lodLevel2;
    public string assetPath = "D:\\Dropbox\\Projects\\metaballs\\";

    public void Start()
    {
        LODGroup lodGroup = gameObject.AddComponent<LODGroup>();
        lodLevel0 = CreateLODLevel(0, 16);
        lodLevel0.transform.parent = gameObject.transform;
        lodLevel1 = CreateLODLevel(1, 16);
        lodLevel1.transform.parent = gameObject.transform;
        lodLevel2 = CreateLODLevel(2, 8);
        lodLevel2.transform.parent = gameObject.transform;
        lodGroup.SetLODs(new[] { new LOD(0.20f, new[] { lodLevel0.GetComponent<MeshRenderer>() }),
                                 new LOD(0.05f, new[] { lodLevel1.GetComponent<MeshRenderer>() }),
                                 new LOD(0.01f, new[] { lodLevel2.GetComponent<MeshRenderer>() })});
    }

    private GameObject CreateLODLevel(int level, int size)
    {
        var texture = LoadTexture(String.Format(assetPath+"texture_{0}.bin", level), size, TextureValue.UInt8);
        //var normals = LoadTexture(String.Format(assetPath+"normals_{0}.bin", level), size, TextureValue.Float);
        var vertices = LoadPoints(String.Format(assetPath+"vertices_{0}.bin", level), -0.5f, -0.5f, -0.5f);
        var vnormals = LoadPoints(String.Format(assetPath+"vnormals_{0}.bin", level));
        var faces = LoadFaces(String.Format(assetPath+"faces_{0}.bin", level));
        GameObject lodObject = new GameObject(String.Format("LOD_{0}", level));
        MeshRenderer meshRenderer = lodObject.AddComponent<MeshRenderer>();
        Material material = new Material(Shader.Find("Custom/MetaballSurfaceShader"));
        material.SetTexture("_MainTex", texture);
        //material.EnableKeyword("_NORMALMAP");
        //material.SetTexture("_BumpMap", normals);
        meshRenderer.sharedMaterial = material;
        MeshFilter meshFilter = lodObject.AddComponent<MeshFilter>();
        Mesh mesh = new Mesh();
        mesh.vertices = vertices;
        mesh.triangles = faces;
        mesh.normals = vnormals;
        meshFilter.mesh = mesh;
        return lodObject;
    }

    private Texture3D LoadTexture(string filename, int size, TextureValue value)
    {
        Texture3D texture = new Texture3D(size, size, size, TextureFormat.RGBA32, false);
        texture.wrapMode = TextureWrapMode.Clamp;
        if (File.Exists(filename))
        {
            using (BinaryReader reader = new BinaryReader(File.Open(filename, FileMode.Open)))
            {
                Color[] colors = new Color[size * size * size];
                for (int x = 0; x < size; x++)
                {
                    int xOffset = x * size * size;
                    for (int y = 0; y < size; y++)
                    {
                        int yOffset = y * size;
                        for (int z = 0; z < size; z++)
                        {
                            switch (value)
                            {
                                case TextureValue.UInt8:
                                    {
                                        colors[z + yOffset + xOffset] = new Color(reader.ReadByte() / 256f, reader.ReadByte() / 256f, reader.ReadByte() / 256f, reader.ReadByte() / 256f);
                                        break;
                                    }
                                case TextureValue.Int8:
                                    {
                                        colors[z + yOffset + xOffset] = new Color(reader.ReadSByte() / 128f, reader.ReadSByte() / 128f, reader.ReadSByte() / 128f, reader.ReadSByte() / 128f);
                                        break;
                                    }
                                case TextureValue.Float:
                                    {
                                        colors[z + yOffset + xOffset] = new Color(reader.ReadSingle(), reader.ReadSingle(), reader.ReadSingle(), reader.ReadSingle());
                                        break;
                                    }
                                default: break;
                            }
                        }
                    }
                }
                texture.SetPixels(colors);
                texture.filterMode = FilterMode.Bilinear;
                texture.Apply();
            }
        }
        return texture;
    }

    private Vector3[] LoadPoints(string filename, float offsetX = 0f, float offsetY = 0f, float offsetZ = 0f)
    {
        var vertices = new ArrayList();
        if (File.Exists(filename))
        {
            using (BinaryReader reader = new BinaryReader(File.Open(filename, FileMode.Open)))
            {
                while (true)
                {
                    try
                    {
                        vertices.Add(new Vector3(reader.ReadSingle() + offsetX, 
                                                 reader.ReadSingle() + offsetY, 
                                                 reader.ReadSingle() + offsetZ));
                    }
                    catch
                    {
                        break;
                    }
                }
            }
        }
        return vertices.ToArray(typeof(Vector3)) as Vector3[];
    }

    private int[] LoadFaces(string filename)
    {
        var vertices = new ArrayList();
        if (File.Exists(filename))
        {
            using (BinaryReader reader = new BinaryReader(File.Open(filename, FileMode.Open)))
            {
                while (true)
                {
                    try
                    {
                        vertices.Add(reader.ReadInt16());
                    }
                    catch
                    {
                        break;
                    }
                }
            }
        }
        return vertices.ToArray(typeof(int)) as int[];
    }
}