# Metabox
Metabox is a [metaball](https://en.wikipedia.org/wiki/Metaball) based procedural block generator.

## Files
* mesh_metaball.py - procedural LOD assets generator
* LoadObject.cs - unity LOD assets loader script (only for testing purposes)
* MetaballSurfaceShader.shader - HLSL unity surface shader for asset rendering with 3D texture
* pySimplify - mesh simplification library from [https://github.com/Kramer84/Py_Fast-Quadric-Mesh-Simplification](https://github.com/Kramer84/Py_Fast-Quadric-Mesh-Simplification)
   
### Input data: 
1. Uniform 3D mesh of metaball samples, usually 8x8x8 but it is possible to use other sizes,
such as 16x16x16, 32x32x32, etc. Each sample consists fields:  
  scale - value of metaball scale 0 - 255 (0.125 .. 32.0), where 8 is x1 scale.  
  color - RGBA value of metaball color 0 - 255    
2. Color 3D texture size factor. Usually twice (factor=2) as metaball mesh size, e.g. 16 for 8x8x8. 
The texture will be 16x16x16 RGBA32 texture.  

### Output data:  
Binary files:  
*    vertices_X - array of vertices coords (4 bytes floats) for X level of details (x = 0..3)  
*    faces_X - array of face's vertex indexes (2 bytes uint) for X level of details (x = 0..3)  
*    vnormals_X - array of vertexes' normal vector values (4 bytes float) for X level of details (x = 0..3)  
*    texture_X - flattened 3D array of 3D color texture values (4 bytes RGBA uint) for X level of details (x = 0..3)  

## Run
1. Compile pySimplify:  
```shell script
cd pySimplify
python setup.py build_ext --inplace
```
2. Create empty Unity3D project.
3. Import LoadObject.cs and MetaballSurfaceShader.shader.
4. Create empty GameObject and add LoadObject script to it.
5. Patch the asset's path (at the #21 line of LoadObject.cs) to this project home folder if necessary.
6. Run the generator:
```shell script
python mesh_metaball.py
``` 
7. Run the unity project
8. Enjoy
